@@ -18,7 +18,12 @@
 #include "banned.h"
 // Required minimum OpenGL version
 #define OPENGL_REQUIRED_VERSION_MAJOR 3
+#ifdef __APPLE__
 #define OPENGL_REQUIRED_VERSION_MINOR 3
+#else
+#define OPENGL_REQUIRED_VERSION_MINOR 1
+#endif
+#define GLSL_VERSION 140
 #define GLFW_MOD_KITTY (GLFW_MOD_LAST * 2)
 #define UNUSED __attribute__ ((unused))
 #define PYNOARG PyObject *__a1 UNUSED, PyObject *__a2 UNUSED
@@ -76,6 +81,9 @@ typedef struct ImageAnchorPosition {
 #define DECORATION_FG_CODE 58
 #define CHAR_IS_BLANK(ch) ((ch) == 32 || (ch) == 0)
 
+// PUA character used as an image placeholder.
+#define IMAGE_PLACEHOLDER_CHAR 0x10EEEE
+
 #define FG 1
 #define BG 2
 
@@ -187,6 +195,7 @@ typedef union LineAttrs {
     struct {
         uint8_t is_continued : 1;
         uint8_t has_dirty_text : 1;
+        uint8_t has_image_placeholders : 1;
         PromptKind prompt_kind : 2;
     };
     uint8_t val;
@@ -377,7 +386,7 @@ void scroll_event(double, double, int, int);
 void on_key_input(GLFWkeyevent *ev);
 void request_window_attention(id_type, bool);
 #ifndef __APPLE__
-void play_canberra_sound(const char *which_sound, const char *event_id, bool is_path, const char *role);
+void play_canberra_sound(const char *which_sound, const char *event_id, bool is_path, const char *role, const char *theme_name);
 #endif
 SPRITE_MAP_HANDLE alloc_sprite_map(unsigned int, unsigned int);
 SPRITE_MAP_HANDLE free_sprite_map(SPRITE_MAP_HANDLE);
