@@ -15,15 +15,11 @@
 #ifndef __APPLE__
 #include "freetype_render_ui_text.h"
 #endif
-extern bool cocoa_make_window_resizable(void *w, bool);
 extern void cocoa_focus_window(void *w);
 extern long cocoa_window_number(void *w);
 extern void cocoa_create_global_menu(void);
-extern void cocoa_hide_window_title(void *w);
 extern void cocoa_system_beep(const char*);
 extern void cocoa_set_activation_policy(bool);
-extern void cocoa_set_titlebar_appearance(void *w, unsigned int theme);
-extern void cocoa_set_titlebar_color(void *w, color_type color);
 extern bool cocoa_alt_option_key_pressed(unsigned long);
 extern void cocoa_toggle_secure_keyboard_entry(void);
 extern void cocoa_hide(void);
@@ -39,6 +35,14 @@ static GLFWcursor *standard_cursor = NULL, *click_cursor = NULL, *arrow_cursor =
 
 static void set_os_window_dpi(OSWindow *w);
 
+static void
+apply_swap_interval(int val) {
+    (void)val;
+#ifndef __APPLE__
+    if (val < 0) val = OPT(sync_to_monitor) && !global_state.is_wayland ? 1 : 0;
+    glfwSwapInterval(val);
+#endif
+}
 
 void
 get_platform_dependent_config_values(void *glfw_window) {
@@ -285,11 +289,19 @@ window_iconify_callback(GLFWwindow *window, int iconified) {
     global_state.callback_os_window = NULL;
 }
 
+void
+change_live_resize_state(OSWindow *w, bool in_progress) {
+    if (in_progress != w->live_resize.in_progress) {
+        w->live_resize.in_progress = in_progress;
+        apply_swap_interval(in_progress ? 0 : -1);
+    }
+}
+
 static void
 live_resize_callback(GLFWwindow *w, bool started) {
     if (!set_callback_window(w)) return;
     global_state.callback_os_window->live_resize.from_os_notification = true;
-    global_state.callback_os_window->live_resize.in_progress = true;
+    change_live_resize_state(global_state.callback_os_window, true);
     global_state.has_pending_resizes = true;
     if (!started) {
         global_state.callback_os_window->live_resize.os_says_resize_complete = true;
@@ -305,12 +317,12 @@ framebuffer_size_callback(GLFWwindow *w, int width, int height) {
     if (width >= min_width && height >= min_height) {
         OSWindow *window = global_state.callback_os_window;
         global_state.has_pending_resizes = true;
-        window->live_resize.in_progress = true;
+        change_live_resize_state(global_state.callback_os_window, true);
         window->live_resize.last_resize_event_at = monotonic();
         window->live_resize.width = MAX(0, width); window->live_resize.height = MAX(0, height);
         window->live_resize.num_of_resize_events++;
         make_os_window_context_current(window);
-        update_surface_size(width, height, window->offscreen_texture_id);
+        update_surface_size(width, height, 0);
         request_tick_callback();
     } else log_error("Ignoring resize request for tiny size: %dx%d", width, height);
     global_state.callback_os_window = NULL;
@@ -322,7 +334,8 @@ dpi_change_callback(GLFWwindow *w, float x_scale UNUSED, float y_scale UNUSED) {
     // Ensure update_os_window_viewport() is called in the near future, it will
     // take care of DPI changes.
     OSWindow *window = global_state.callback_os_window;
-    window->live_resize.in_progress = true; global_state.has_pending_resizes = true;
+    change_live_resize_state(global_state.callback_os_window, true);
+    global_state.has_pending_resizes = true;
     window->live_resize.last_resize_event_at = monotonic();
     global_state.callback_os_window = NULL;
     request_tick_callback();
@@ -827,16 +840,87 @@ intercept_cocoa_fullscreen(GLFWwindow *w) {
 }
 #endif
 
-void
-set_titlebar_color(OSWindow *w, color_type color, bool use_system_color, unsigned int system_color UNUSED) {
-    if (w->handle && (!w->last_titlebar_color || (w->last_titlebar_color & 0xffffff) != (color & 0xffffff))) {
-        w->last_titlebar_color = (1 << 24) | (color & 0xffffff);
+static void
+init_window_chrome_state(WindowChromeState *s, color_type active_window_bg, bool is_semi_transparent, float background_opacity) {
+    zero_at_ptr(s);
+    const bool should_blur = background_opacity < 1.f && OPT(background_blur) > 0 && is_semi_transparent;
+#define SET_TCOL(val) \
+        s->use_system_color = false; \
+        switch (val & 0xff) { \
+            case 0: s->use_system_color = true; break; \
+            case 1: s->color = active_window_bg; break; \
+            default: s->color = val >> 8; break; \
+        }
+
 #ifdef __APPLE__
-        if (!use_system_color) cocoa_set_titlebar_color(glfwGetCocoaWindow(w->handle), color);
-        else cocoa_set_titlebar_appearance(glfwGetCocoaWindow(w->handle), system_color);
+    if (OPT(macos_titlebar_color) < 0) {
+        s->use_system_color = true;
+        s->system_color = -OPT(macos_titlebar_color);
+    } else {
+        unsigned long val = OPT(macos_titlebar_color);
+        SET_TCOL(val);
+    }
+    s->macos_colorspace = OPT(macos_colorspace);
+    s->resizable = OPT(macos_window_resizable);
 #else
-        if (global_state.is_wayland && glfwWaylandSetTitlebarColor) glfwWaylandSetTitlebarColor(w->handle, color, use_system_color);
+    if (global_state.is_wayland) { SET_TCOL(OPT(wayland_titlebar_color)); }
 #endif
+    s->background_blur = should_blur ? OPT(background_blur) : 0;
+    s->hide_window_decorations = OPT(hide_window_decorations);
+    s->show_title_in_titlebar = (OPT(macos_show_window_title_in) & WINDOW) != 0;
+    s->background_opacity = background_opacity;
+}
+
+#ifdef __APPLE__
+static void
+apply_window_chrome_state(GLFWwindow *w, WindowChromeState new_state, int width, int height, bool window_decorations_changed) {
+    glfwCocoaSetWindowChrome(w,
+        new_state.color, new_state.use_system_color, new_state.system_color,
+        new_state.background_blur, new_state.hide_window_decorations,
+        new_state.show_title_in_titlebar, new_state.macos_colorspace,
+        new_state.background_opacity, new_state.resizable
+    );
+    // Need to resize the window again after hiding decorations or title bar to take up screen space
+    if (window_decorations_changed) glfwSetWindowSize(w, width, height);
+}
+#endif
+
+void
+set_os_window_chrome(OSWindow *w) {
+    if (!w->handle) return;
+    color_type bg = OPT(background);
+    if (w->num_tabs > w->active_tab) {
+        Tab *tab = w->tabs + w->active_tab;
+        if (tab->num_windows > tab->active_window) {
+            Window *window = tab->windows + tab->active_window;
+            ColorProfile *c;
+            if (window->render_data.screen && (c=window->render_data.screen->color_profile)) {
+                bg = colorprofile_to_color(c, c->overridden.default_bg, c->configured.default_bg).rgb;
+            }
+        }
+    }
+
+    WindowChromeState new_state;
+    init_window_chrome_state(&new_state, bg, w->is_semi_transparent, w->background_opacity);
+    if (memcmp(&new_state, &w->last_window_chrome, sizeof(WindowChromeState)) != 0) {
+        int width, height;
+        glfwGetWindowSize(w->handle, &width, &height);
+        bool window_decorations_changed = new_state.hide_window_decorations != w->last_window_chrome.hide_window_decorations;
+#ifdef __APPLE__
+        apply_window_chrome_state(w->handle, new_state, width, height, window_decorations_changed);
+#else
+        if (window_decorations_changed) {
+            bool hide_window_decorations = new_state.hide_window_decorations & 1;
+            glfwSetWindowAttrib(w->handle, GLFW_DECORATED, !hide_window_decorations);
+            glfwSetWindowSize(w->handle, width, height);
+        }
+        if (global_state.is_wayland) {
+            if (glfwWaylandSetTitlebarColor) glfwWaylandSetTitlebarColor(w->handle, new_state.color, new_state.use_system_color);
+        } else {
+            glfwSetX11WindowBlurred(w->handle, new_state.background_blur > 0);
+        }
+#endif
+        w->last_window_chrome = new_state;
     }
 }
 
@@ -869,7 +953,6 @@ create_os_window(PyObject UNUSED *self, PyObject *args, PyObject *kw) {
         // We don't use depth and stencil buffers
         glfwWindowHint(GLFW_DEPTH_BITS, 0);
         glfwWindowHint(GLFW_STENCIL_BITS, 0);
-        if (OPT(hide_window_decorations) & 1) glfwWindowHint(GLFW_DECORATED, false);
         glfwSetApplicationCloseCallback(application_close_requested_callback);
         glfwSetCurrentSelectionCallback(get_current_selection);
         glfwSetHasCurrentSelectionCallback(has_current_selection);
@@ -882,10 +965,18 @@ create_os_window(PyObject UNUSED *self, PyObject *args, PyObject *kw) {
         glfwSetApplicationWillFinishLaunching(cocoa_create_global_menu);
 #endif
     }
+    if (OPT(hide_window_decorations) & 1) glfwWindowHint(GLFW_DECORATED, false);
 
+    const bool set_blur = OPT(background_blur) > 0 && OPT(background_opacity) < 1.f;
 #ifdef __APPLE__
     glfwWindowHint(GLFW_COCOA_COLOR_SPACE, OPT(macos_colorspace));
+    if (set_blur) {
+        glfwWindowHint(GLFW_COCOA_BLUR_RADIUS, MIN(OPT(background_blur), 128));
+    } else {
+        glfwWindowHint(GLFW_COCOA_BLUR_RADIUS, 0);
+    }
 #else
+    if (!global_state.is_wayland) glfwWindowHint(GLFW_X11_BLUR, set_blur);
     glfwWindowHintString(GLFW_X11_INSTANCE_NAME, wm_class_name);
     glfwWindowHintString(GLFW_X11_CLASS_NAME, wm_class_class);
     glfwWindowHintString(GLFW_WAYLAND_APP_ID, wm_class_class);
@@ -940,9 +1031,7 @@ create_os_window(PyObject UNUSED *self, PyObject *args, PyObject *kw) {
     // blank the window once so that there is no initial flash of color
     // changing, in case the background color is not black
     blank_canvas(is_semi_transparent ? OPT(background_opacity) : 1.0f, OPT(background));
-#ifndef __APPLE__
-    glfwSwapInterval(OPT(sync_to_monitor) && !global_state.is_wayland ? 1 : 0);
-#endif
+    apply_swap_interval(-1);
     // On Wayland the initial swap is allowed only after the first XDG configure event
     if (glfwAreSwapsAllowed(glfw_window)) glfwSwapBuffers(glfw_window);
     glfwSetInputMode(glfw_window, GLFW_LOCK_KEY_MODS, true);
@@ -990,18 +1079,6 @@ create_os_window(PyObject UNUSED *self, PyObject *args, PyObject *kw) {
     w->fonts_data = fonts_data;
     w->shown_once = true;
     w->last_focused_counter = ++focus_counter;
-#ifdef __APPLE__
-    if (OPT(hide_window_decorations) & 2) {
-        glfwHideCocoaTitlebar(glfw_window, true);
-    } else if (!(OPT(macos_show_window_title_in) & WINDOW)) {
-        if (glfwGetCocoaWindow) cocoa_hide_window_title(glfwGetCocoaWindow(glfw_window));
-        else log_error("Failed to load glfwGetCocoaWindow");
-    }
-    if (OPT(hide_window_decorations)) {
-        // Need to resize the window again after hiding decorations or title bar to take up screen space
-        glfwSetWindowSize(glfw_window, width, height);
-    }
-#endif
     os_window_update_size_increments(w);
 #ifdef __APPLE__
     if (OPT(macos_option_as_alt)) glfwSetCocoaTextInputFilter(glfw_window, filter_option);
@@ -1011,9 +1088,6 @@ create_os_window(PyObject UNUSED *self, PyObject *args, PyObject *kw) {
     if (logo.pixels && logo.width && logo.height) glfwSetWindowIcon(glfw_window, 1, &logo);
     glfwSetCursor(glfw_window, standard_cursor);
     update_os_window_viewport(w, false);
-#ifdef __APPLE__
-    if (glfwGetCocoaWindow) cocoa_make_window_resizable(glfwGetCocoaWindow(glfw_window), OPT(macos_window_resizable));
-#endif
     glfwSetWindowPosCallback(glfw_window, window_pos_callback);
     // missing size callback
     glfwSetWindowCloseCallback(glfw_window, window_close_callback);
@@ -1044,6 +1118,10 @@ create_os_window(PyObject UNUSED *self, PyObject *args, PyObject *kw) {
             warned = true;
         }
     }
+    init_window_chrome_state(&w->last_window_chrome, OPT(background), w->is_semi_transparent, w->background_opacity);
+#ifdef __APPLE__
+    apply_window_chrome_state(w->handle, w->last_window_chrome, width, height, OPT(hide_window_decorations) != 0);
+#endif
     // Update window state
     // We do not call glfwWindowHint to set GLFW_MAXIMIZED before the window is created.
     // That would cause the window to be set to maximize immediately after creation and use the wrong initial size when restored.
@@ -1808,7 +1886,7 @@ make_x11_window_a_dock_window(PyObject *self UNUSED, PyObject *args UNUSED) {
     int x11_window_id;
     PyObject *dims;
     if (!PyArg_ParseTuple(args, "iO!", &x11_window_id, &PyTuple_Type, &dims)) return NULL;
-    if (PyTuple_GET_SIZE(dims) != 12 ) { PyErr_SetString(PyExc_TypeError, "dimesions must be a tuple of length 12"); return NULL; }
+    if (PyTuple_GET_SIZE(dims) != 12 ) { PyErr_SetString(PyExc_TypeError, "dimensions must be a tuple of length 12"); return NULL; }
     if (!glfwSetX11WindowAsDock) { PyErr_SetString(PyExc_RuntimeError, "Failed to load glfwGetX11Window"); return NULL; }
     uint32_t dimensions[12];
     for (Py_ssize_t i = 0; i < 12; i++) dimensions[i] = PyLong_AsUnsignedLong(PyTuple_GET_ITEM(dims, i));
