@@ -49,19 +49,19 @@ from .fonts.render import set_font_family
 from .options.types import Options
 from .options.utils import DELETE_ENV_VAR
 from .os_window_size import initial_window_size_func
-from .prewarm import PrewarmProcess, fork_prewarm_process
 from .session import create_sessions, get_os_window_sizing_data
+from .shaders import CompileError, load_shader_programs
 from .types import SingleInstanceData
 from .utils import (
     cleanup_ssh_control_masters,
     detach,
     expandvars,
     log_error,
+    parse_os_window_state,
     single_instance,
     startup_notification_handler,
     unix_socket_paths,
 )
-from .window import load_shader_programs
 
 
 def set_custom_ibeam_cursor() -> None:
@@ -85,6 +85,8 @@ def talk_to_instance(args: CLIOptions) -> None:
     session_data = ''
     if args.session == '-':
         session_data = sys.stdin.read()
+    elif args.session == 'none':
+        session_data = 'none'
     elif args.session:
         with open(args.session) as f:
             session_data = f.read()
@@ -123,8 +125,11 @@ def talk_to_instance(args: CLIOptions) -> None:
 
 
 def load_all_shaders(semi_transparent: bool = False) -> None:
-    load_shader_programs(semi_transparent)
-    load_borders_program()
+    try:
+        load_shader_programs(semi_transparent)
+        load_borders_program()
+    except CompileError as err:
+        raise SystemExit(err)
 
 
 def init_glfw_module(glfw_module: str, debug_keyboard: bool = False, debug_rendering: bool = False) -> None:
@@ -143,7 +148,7 @@ def get_macos_shortcut_for(
 ) -> Optional[SingleKey]:
     # for maximum robustness we should use opts.alias_map to resolve
     # aliases however this requires parsing everything on startup which could
-    # be potentially slow. Lets just hope the user doesnt alias these
+    # be potentially slow. Lets just hope the user doesn't alias these
     # functions.
     ans = None
     candidates = []
@@ -189,7 +194,7 @@ def set_macos_app_custom_icon() -> None:
                     log_error('Failed to set custom app icon, ignoring')
             # macOS Dock does not reload icons until it is restarted, so we set
             # the application icon here. This will revert when kitty quits, but
-            # cant be helped since there appears to be no way to get the dock
+            # can't be helped since there appears to be no way to get the dock
             # to reload short of killing it.
             cocoa_set_dock_icon(icon_path)
             break
@@ -201,7 +206,7 @@ def set_x11_window_icon() -> None:
     set_default_window_icon(f'{path}-128{ext}')
 
 
-def _run_app(opts: Options, args: CLIOptions, prewarm: PrewarmProcess, bad_lines: Sequence[BadLine] = ()) -> None:
+def _run_app(opts: Options, args: CLIOptions, bad_lines: Sequence[BadLine] = ()) -> None:
     global_shortcuts: Dict[str, SingleKey] = {}
     if is_macos:
         from collections import defaultdict
@@ -238,13 +243,17 @@ def _run_app(opts: Options, args: CLIOptions, prewarm: PrewarmProcess, bad_lines
     with cached_values_for(run_app.cached_values_name) as cached_values:
         startup_sessions = tuple(create_sessions(opts, args, default_session=opts.startup_session))
         wincls = (startup_sessions[0].os_window_class if startup_sessions else '') or args.cls or appname
+        window_state = (args.start_as if args.start_as and args.start_as != 'normal' else None) or (
+            getattr(startup_sessions[0], 'os_window_state', None) if startup_sessions else None
+        )
+        wstate = parse_os_window_state(window_state) if window_state is not None else None
         with startup_notification_handler(extra_callback=run_app.first_window_callback) as pre_show_callback:
             window_id = create_os_window(
                     run_app.initial_window_size_func(get_os_window_sizing_data(opts, startup_sessions[0] if startup_sessions else None), cached_values),
                     pre_show_callback,
                     args.title or appname, args.name or args.cls or appname,
-                    wincls, load_all_shaders, disallow_override_title=bool(args.title))
-        boss = Boss(opts, args, cached_values, global_shortcuts, prewarm)
+                    wincls, wstate, load_all_shaders, disallow_override_title=bool(args.title))
+        boss = Boss(opts, args, cached_values, global_shortcuts)
         boss.start(window_id, startup_sessions)
         if bad_lines:
             boss.show_bad_config_lines(bad_lines)
@@ -261,12 +270,12 @@ class AppRunner:
         self.first_window_callback = lambda window_handle: None
         self.initial_window_size_func = initial_window_size_func
 
-    def __call__(self, opts: Options, args: CLIOptions, prewarm: PrewarmProcess, bad_lines: Sequence[BadLine] = ()) -> None:
+    def __call__(self, opts: Options, args: CLIOptions, bad_lines: Sequence[BadLine] = ()) -> None:
         set_scale(opts.box_drawing_scale)
         set_options(opts, is_wayland(), args.debug_rendering, args.debug_font_fallback)
         try:
             set_font_family(opts, debug_font_matching=args.debug_font_fallback)
-            _run_app(opts, args, prewarm, bad_lines)
+            _run_app(opts, args, bad_lines)
         finally:
             set_options(None)
             free_font_data()  # must free font data before glfw/freetype/fontconfig/opengl etc are finalized
@@ -285,15 +294,20 @@ def ensure_macos_locale() -> None:
     # https://github.com/kovidgoyal/kitty/issues/90
     from .fast_data_types import cocoa_get_lang, locale_is_valid
     if 'LANG' not in os.environ:
-        lang = cocoa_get_lang()
-        if lang is not None:
-            if not locale_is_valid(lang):
-                if lang.startswith('en_'):
-                    lang = 'en_US'
-                else:
-                    log_error(f'Could not set LANG Cocoa returns language as: {lang}')
-            os.environ['LANG'] = f'{lang}.UTF-8'
-            set_LANG_in_default_env(os.environ['LANG'])
+        lang_code, country_code, identifier = cocoa_get_lang()
+        lang = 'en_US'
+        if identifier and locale_is_valid(identifier):
+            lang = identifier
+        elif lang_code and country_code and locale_is_valid(f'{lang_code}_{country_code}'):
+            lang = f'{lang_code}_{country_code}'
+        elif lang_code:
+            if lang_code != 'en':
+                with suppress(OSError):
+                    found = sorted(x for x in os.listdir('/usr/share/locale') if x.startswith(f'{lang_code}_'))
+                    if found:
+                        lang = found[0].partition('.')[0]
+        os.environ['LANG'] = f'{lang}.UTF-8'
+        set_LANG_in_default_env(os.environ['LANG'])
 
 
 @contextmanager
@@ -492,9 +506,6 @@ def _main() -> None:
     bad_lines: List[BadLine] = []
     opts = create_opts(cli_opts, accumulate_bad_lines=bad_lines)
     setup_environment(opts, cli_opts)
-    prewarm = fork_prewarm_process(opts)
-    if prewarm is None:
-        raise SystemExit(1)
 
     # set_locale on macOS uses cocoa APIs when LANG is not set, so we have to
     # call it after the fork
@@ -517,7 +528,7 @@ def _main() -> None:
     try:
         with setup_profiling():
             # Avoid needing to launch threads to reap zombies
-            run_app(opts, cli_opts, prewarm, bad_lines)
+            run_app(opts, cli_opts, bad_lines)
     finally:
         glfw_terminate()
         cleanup_ssh_control_masters()
