@@ -280,7 +280,7 @@ def remote_control(func: str, rest: str) -> FuncArgsType:
     return func, args
 
 
-@func_with_args('nth_os_window', 'nth_window', 'scroll_to_prompt', 'visual_window_select_action_trigger')
+@func_with_args('nth_os_window', 'nth_window', 'scroll_to_prompt', 'visual_window_select_action_trigger', 'next_layout')
 def single_integer_arg(func: str, rest: str) -> FuncArgsType:
     try:
         num = int(rest)
@@ -604,6 +604,13 @@ def resize_draw_strategy(x: str) -> int:
     return cmap.get(x.lower(), 0)
 
 
+def resize_debounce_time(x: str) -> Tuple[float, float]:
+    parts = x.split(maxsplit=1)
+    if len(parts) == 1:
+        return positive_float(parts[0]), 0.5
+    return positive_float(parts[0]), positive_float(parts[1])
+
+
 def visual_window_select_characters(x: str) -> str:
     import string
     valid_characters = string.digits + string.ascii_uppercase
@@ -854,12 +861,14 @@ def store_multiple(val: str, current_val: Container[str]) -> Iterable[Tuple[str,
         yield val, val
 
 
+allowed_shell_integration_values = frozenset({'enabled', 'disabled', 'no-rc', 'no-cursor', 'no-title', 'no-prompt-mark', 'no-complete', 'no-cwd'})
+
+
 def shell_integration(x: str) -> FrozenSet[str]:
-    s = frozenset({'enabled', 'disabled', 'no-rc', 'no-cursor', 'no-title', 'no-prompt-mark', 'no-complete', 'no-cwd'})
     q = frozenset(x.lower().split())
-    if not q.issubset(s):
-        log_error(f'Invalid shell integration options: {q - s}, ignoring')
-        return q & s or frozenset({'invalid'})
+    if not q.issubset(allowed_shell_integration_values):
+        log_error(f'Invalid shell integration options: {q - allowed_shell_integration_values}, ignoring')
+        return q & allowed_shell_integration_values or frozenset({'invalid'})
     return q
 
 
