@@ -203,8 +203,8 @@ add_os_window(void) {
     zero_at_ptr(ans);
     ans->id = ++global_state.os_window_id_counter;
     ans->tab_bar_render_data.vao_idx = create_cell_vao();
-    ans->gvao_idx = create_graphics_vao();
     ans->background_opacity = OPT(background_opacity);
+    ans->created_at = monotonic();
 
     bool wants_bg = OPT(background_image) && OPT(background_image)[0] != 0;
     if (wants_bg) {
@@ -244,15 +244,12 @@ add_tab(id_type os_window_id) {
 static void
 create_gpu_resources_for_window(Window *w) {
     w->render_data.vao_idx = create_cell_vao();
-    w->render_data.gvao_idx = create_graphics_vao();
 }
 
 static void
 release_gpu_resources_for_window(Window *w) {
     if (w->render_data.vao_idx > -1) remove_vao(w->render_data.vao_idx);
     w->render_data.vao_idx = -1;
-    if (w->render_data.gvao_idx > -1) remove_vao(w->render_data.gvao_idx);
-    w->render_data.gvao_idx = -1;
 }
 
 static bool
@@ -292,7 +289,6 @@ initialize_window(Window *w, PyObject *title, bool init_gpu_resources) {
     if (init_gpu_resources) create_gpu_resources_for_window(w);
     else {
         w->render_data.vao_idx = -1;
-        w->render_data.gvao_idx = -1;
     }
 }
 
@@ -477,10 +473,7 @@ destroy_os_window_item(OSWindow *w) {
         remove_tab_inner(w, tab->id);
     }
     Py_CLEAR(w->window_title); Py_CLEAR(w->tab_bar_render_data.screen);
-    if (w->offscreen_texture_id) free_texture(&w->offscreen_texture_id);
-    if (w->offscreen_framebuffer) free_framebuffer(&w->offscreen_framebuffer);
     remove_vao(w->tab_bar_render_data.vao_idx);
-    remove_vao(w->gvao_idx);
     free(w->tabs); w->tabs = NULL;
     free_bgimage(&w->bgimage, true);
     w->bgimage = NULL;
@@ -523,6 +516,7 @@ set_active_window(id_type os_window_id, id_type tab_id, id_type window_id) {
         (void)window;
         tab->active_window = w;
         osw->needs_render = true;
+        set_os_window_chrome(osw);
     END_WITH_WINDOW;
 }
 
@@ -884,14 +878,11 @@ PYWRAP1(run_with_activation_token) {
     Py_RETURN_NONE;
 }
 
-PYWRAP1(set_titlebar_color) {
+PYWRAP1(set_os_window_chrome) {
     id_type os_window_id;
-    unsigned int color;
-    int use_system_color = 0;
-    unsigned int system_color = 0;
-    PA("KI|pI", &os_window_id, &color, &use_system_color, &system_color);
+    PA("K", &os_window_id);
     WITH_OS_WINDOW(os_window_id)
-        set_titlebar_color(os_window, color, use_system_color, system_color);
+        set_os_window_chrome(os_window);
         Py_RETURN_TRUE;
     END_WITH_OS_WINDOW
     Py_RETURN_FALSE;
@@ -946,7 +937,6 @@ PYWRAP1(set_window_render_data) {
     WITH_WINDOW(os_window_id, tab_id, window_id);
         Py_CLEAR(window->render_data.screen);
         d.vao_idx = window->render_data.vao_idx;
-        d.gvao_idx = window->render_data.gvao_idx;
         init_screen_render_data(osw, &g, &d);
         window->render_data = d;
         window->geometry = g;
@@ -1370,7 +1360,7 @@ static PyMethodDef module_methods[] = {
     MW(mark_os_window_for_close, METH_VARARGS),
     MW(set_application_quit_request, METH_VARARGS),
     MW(current_application_quit_request, METH_NOARGS),
-    MW(set_titlebar_color, METH_VARARGS),
+    MW(set_os_window_chrome, METH_VARARGS),
     MW(focus_os_window, METH_VARARGS),
     MW(mark_tab_bar_dirty, METH_O),
     MW(run_with_activation_token, METH_O),
