@@ -7,7 +7,7 @@ import re
 import sys
 import weakref
 from collections import deque
-from contextlib import suppress
+from contextlib import contextmanager, suppress
 from enum import Enum, IntEnum, auto
 from functools import lru_cache, partial
 from gettext import gettext as _
@@ -19,6 +19,7 @@ from typing import (
     Callable,
     Deque,
     Dict,
+    Generator,
     Iterable,
     List,
     NamedTuple,
@@ -37,38 +38,19 @@ from .constants import (
     appname,
     clear_handled_signals,
     config_dir,
-    is_macos,
     wakeup_io_loop,
 )
 from .fast_data_types import (
-    BGIMAGE_PROGRAM,
-    BLIT_PROGRAM,
-    CELL_BG_PROGRAM,
-    CELL_FG_PROGRAM,
-    CELL_PROGRAM,
-    CELL_SPECIAL_PROGRAM,
     CURSOR_BEAM,
     CURSOR_BLOCK,
     CURSOR_UNDERLINE,
     DCS,
-    DECORATION,
-    DECORATION_MASK,
-    DIM,
     GLFW_MOD_CONTROL,
-    GRAPHICS_ALPHA_MASK_PROGRAM,
-    GRAPHICS_PREMULT_PROGRAM,
-    GRAPHICS_PROGRAM,
-    MARK,
-    MARK_MASK,
     NO_CURSOR_SHAPE,
-    NUM_UNDERLINE_STYLES,
     OSC,
-    REVERSE,
     SCROLL_FULL,
     SCROLL_LINE,
     SCROLL_PAGE,
-    STRIKETHROUGH,
-    TINT_PROGRAM,
     Color,
     KeyEvent,
     Screen,
@@ -77,19 +59,16 @@ from .fast_data_types import (
     cell_size_for_window,
     click_mouse_cmd_output,
     click_mouse_url,
-    compile_program,
     current_focused_os_window_id,
     encode_key_for_tty,
     get_boss,
     get_click_interval,
     get_options,
-    init_cell_program,
     last_focused_os_window_id,
     mark_os_window_dirty,
     mouse_selection,
     move_cursor_to_mouse_if_in_prompt,
     pt_to_px,
-    set_titlebar_color,
     set_window_logo,
     set_window_padding,
     set_window_render_data,
@@ -112,7 +91,6 @@ from .typing import BossType, ChildType, EdgeLiteral, TabType, TypedDict
 from .utils import (
     docs_url,
     kitty_ansi_sanitizer_pat,
-    load_shaders,
     log_error,
     open_cmd,
     open_url,
@@ -144,6 +122,7 @@ class CwdRequest:
     def __init__(self, window: Optional['Window'] = None, request_type: CwdRequestType = CwdRequestType.current) -> None:
         self.window_id = -1 if window is None else window.id
         self.request_type = request_type
+        self.rc_from_window_id = 0
 
     def __bool__(self) -> bool:
         return self.window_id > -1
@@ -170,14 +149,15 @@ class CwdRequest:
             return ''
         reported_cwd = path_from_osc7_url(window.screen.last_reported_cwd) if window.screen.last_reported_cwd else ''
         if reported_cwd and (self.request_type is not CwdRequestType.root or window.root_in_foreground_processes):
-            # First check if we are running ssh kitten, and trying to open the configured login shell
-            if argv[0] == resolved_shell(get_options())[0]:
-                ssh_kitten_cmdline = window.ssh_kitten_cmdline()
-                if ssh_kitten_cmdline:
-                    from kittens.ssh.utils import set_cwd_in_cmdline
-                    argv[:] = ssh_kitten_cmdline
-                    set_cwd_in_cmdline(reported_cwd, argv)
-                    return ''
+            ssh_kitten_cmdline = window.ssh_kitten_cmdline()
+            if ssh_kitten_cmdline:
+                run_shell = argv[0] == resolved_shell(get_options())[0]
+                server_args = [] if run_shell else list(argv)
+                from kittens.ssh.utils import set_cwd_in_cmdline, set_server_args_in_cmdline
+                argv[:] = ssh_kitten_cmdline
+                set_cwd_in_cmdline(reported_cwd, argv)
+                set_server_args_in_cmdline(server_args, argv, allocate_tty=not run_shell)
+                return ''
             if not window.child_is_remote and (self.request_type is CwdRequestType.last_reported or window.at_prompt):
                 return reported_cwd
         return window.get_cwd_of_child(oldest=self.request_type is CwdRequestType.oldest) or ''
@@ -360,63 +340,6 @@ def as_text(
     return ans
 
 
-def multi_replace(src: str, **replacements: Any) -> str:
-    r = {k: str(v) for k, v in replacements.items()}
-
-    def sub(m: 're.Match[str]') -> str:
-        return r.get(m.group(1), m.group(1))
-
-    return re.sub(r'\{([A-Z_]+)\}', sub, src)
-
-
-class LoadShaderPrograms:
-
-    def __call__(self, semi_transparent: bool = False) -> None:
-        compile_program(BLIT_PROGRAM, *load_shaders('blit'))
-        v, f = load_shaders('cell')
-
-        for which, p in {
-                'SIMPLE': CELL_PROGRAM,
-                'BACKGROUND': CELL_BG_PROGRAM,
-                'SPECIAL': CELL_SPECIAL_PROGRAM,
-                'FOREGROUND': CELL_FG_PROGRAM,
-        }.items():
-            ff = f.replace('{WHICH_PROGRAM}', which)
-            vv = multi_replace(
-                v,
-                WHICH_PROGRAM=which,
-                REVERSE_SHIFT=REVERSE,
-                STRIKE_SHIFT=STRIKETHROUGH,
-                DIM_SHIFT=DIM,
-                DECORATION_SHIFT=DECORATION,
-                MARK_SHIFT=MARK,
-                MARK_MASK=MARK_MASK,
-                DECORATION_MASK=DECORATION_MASK,
-                STRIKE_SPRITE_INDEX=NUM_UNDERLINE_STYLES + 1,
-            )
-            if semi_transparent:
-                vv = vv.replace('#define NOT_TRANSPARENT', '#define TRANSPARENT')
-                ff = ff.replace('#define NOT_TRANSPARENT', '#define TRANSPARENT')
-            compile_program(p, vv, ff)
-
-        v, f = load_shaders('graphics')
-        for which, p in {
-                'SIMPLE': GRAPHICS_PROGRAM,
-                'PREMULT': GRAPHICS_PREMULT_PROGRAM,
-                'ALPHA_MASK': GRAPHICS_ALPHA_MASK_PROGRAM,
-        }.items():
-            ff = f.replace('ALPHA_TYPE', which)
-            compile_program(p, v, ff)
-
-        v, f = load_shaders('bgimage')
-        compile_program(BGIMAGE_PROGRAM, v, f)
-        v, f = load_shaders('tint')
-        compile_program(TINT_PROGRAM, v, f)
-        init_cell_program()
-
-
-load_shader_programs = LoadShaderPrograms()
-
 
 def setup_colors(screen: Screen, opts: Options) -> None:
     screen.color_profile.update_ansi_color_table(build_ansi_color_table(opts))
@@ -506,6 +429,9 @@ class EdgeWidths:
     def serialize(self) -> Dict[str, Optional[float]]:
         return {'left': self.left, 'right': self.right, 'top': self.top, 'bottom': self.bottom}
 
+    def copy(self) -> 'EdgeWidths':
+        return EdgeWidths(self.serialize())
+
 
 class GlobalWatchers:
 
@@ -537,6 +463,22 @@ class Window:
 
     window_custom_type: str = ''
     overlay_type = OverlayType.transient
+    initial_ignore_focus_changes: bool = False
+    initial_ignore_focus_changes_context_manager_in_operation: bool = False
+
+    @classmethod
+    @contextmanager
+    def set_ignore_focus_changes_for_new_windows(cls, value: bool = True) -> Generator[None, None, None]:
+        if cls.initial_ignore_focus_changes_context_manager_in_operation:
+            yield
+        else:
+            orig, cls.initial_ignore_focus_changes = cls.initial_ignore_focus_changes, value
+            cls.initial_ignore_focus_changes_context_manager_in_operation = True
+            try:
+                yield
+            finally:
+                cls.initial_ignore_focus_changes = orig
+                cls.initial_ignore_focus_changes_context_manager_in_operation = False
 
     def __init__(
         self,
@@ -555,6 +497,7 @@ class Window:
         else:
             self.watchers = global_watchers().copy()
         self.last_focused_at = 0.
+        self.is_focused: bool = False
         self.last_resized_at = 0.
         self.started_at = monotonic()
         self.current_remote_data: List[str] = []
@@ -569,10 +512,12 @@ class Window:
         self.pty_resized_once = False
         self.last_reported_pty_size = (-1, -1, -1, -1)
         self.needs_attention = False
+        self.ignore_focus_changes = self.initial_ignore_focus_changes
         self.override_title = override_title
         self.default_title = os.path.basename(child.argv[0] or appname)
         self.child_title = self.default_title
         self.title_stack: Deque[str] = deque(maxlen=10)
+        self.user_vars: Dict[str, bytes] = {}
         self.id: int = add_window(tab.os_window_id, tab.id, self.title)
         self.clipboard_request_manager = ClipboardRequestManager(self.id)
         self.margin = EdgeWidths()
@@ -672,10 +617,9 @@ class Window:
             val = round(val)
         return int(val)
 
-    def apply_options(self) -> None:
+    def apply_options(self, is_active: bool) -> None:
         opts = get_options()
         self.update_effective_padding()
-        self.change_titlebar_color()
         setup_colors(self.screen, opts)
 
     @property
@@ -814,8 +758,10 @@ class Window:
             if val:
                 self.refresh()
 
-    def refresh(self) -> None:
+    def refresh(self, reload_all_gpu_data: bool = False) -> None:
         self.screen.mark_as_dirty()
+        if reload_all_gpu_data:
+            self.screen.reload_all_gpu_data()
         wakeup_io_loop()
         wakeup_main_loop()
 
@@ -896,7 +842,27 @@ class Window:
         self.override_title = title or None
         self.title_updated()
 
+    def set_user_var(self, key: str, val: Optional[bytes]) -> None:
+        self.user_vars.pop(key, None)  # ensure key will be newest in user_vars even if already present
+        if len(self.user_vars) > 64:  # dont store too many user vars
+            oldest_key = next(iter(self.user_vars))
+            self.user_vars.pop(oldest_key)
+        if val is not None:
+            self.user_vars[key] = val
+
+    # screen callbacks {{{
+
+    def osc_1337(self, raw_data: str) -> None:
+        for record in raw_data.split(';'):
+            key, _, val = record.partition('=')
+            if key == 'SetUserVar':
+                from base64 import standard_b64decode
+                ukey, has_equal, uval = val.partition('=')
+                self.set_user_var(ukey, (standard_b64decode(uval) if uval else b'') if has_equal == '=' else None)
+
     def desktop_notify(self, osc_code: int, raw_data: str) -> None:
+        if osc_code == 1337:
+            self.osc_1337(raw_data)
         if osc_code == 777:
             if not raw_data.startswith('notify;'):
                 log_error(f'Ignoring unknown OSC 777: {raw_data}')
@@ -906,7 +872,6 @@ class Window:
         if cmd is not None and osc_code == 99:
             self.prev_osc99_cmd = cmd
 
-    # screen callbacks {{{
     def use_utf8(self, on: bool) -> None:
         get_boss().child_monitor.set_iutf8_winid(self.id, on)
 
@@ -957,7 +922,7 @@ class Window:
 
     def handle_remote_file(self, netloc: str, remote_path: str) -> None:
         from kittens.remote_file.main import is_ssh_kitten_sentinel
-        from kittens.ssh.main import get_connection_data
+        from kittens.ssh.utils import get_connection_data
 
         from .utils import SSHConnectionData
         args = self.ssh_kitten_cmdline()
@@ -978,7 +943,7 @@ class Window:
             '--ssh-connection-data', json.dumps(conn_data)
         )
 
-    def send_signal_for_key(self, key_num: int) -> bool:
+    def send_signal_for_key(self, key_num: bytes) -> bool:
         try:
             return self.child.send_signal_for_key(key_num)
         except OSError as err:
@@ -986,8 +951,9 @@ class Window:
             return False
 
     def focus_changed(self, focused: bool) -> None:
-        if self.destroyed:
+        if self.destroyed or self.ignore_focus_changes or self.is_focused == focused:
             return
+        self.is_focused = focused
         call_watchers(weakref.ref(self), 'on_focus_change', {'focused': focused})
         for c in self.actions_on_focus_change:
             try:
@@ -1049,18 +1015,6 @@ class Window:
                     tab.relayout_borders()
                 tab.on_bell(self)
 
-    def change_titlebar_color(self) -> None:
-        opts = get_options()
-        val = opts.macos_titlebar_color if is_macos else opts.wayland_titlebar_color
-        if val > 0:
-            if (val & 0xff) == 1:
-                val = self.screen.color_profile.default_bg
-            else:
-                val = val >> 8
-            set_titlebar_color(self.os_window_id, val)
-        else:
-            set_titlebar_color(self.os_window_id, 0, True, -val)
-
     def change_colors(self, changes: Dict[DynamicColor, Optional[str]]) -> None:
         dirtied = default_bg_changed = False
 
@@ -1154,7 +1108,7 @@ class Window:
         self.write_to_child(data)
 
     def handle_remote_ssh(self, msg: str) -> None:
-        from kittens.ssh.main import get_ssh_data
+        from kittens.ssh.utils import get_ssh_data
         for line in get_ssh_data(msg, f'{os.getpid()}-{self.id}'):
             self.write_to_child(line)
 
@@ -1445,9 +1399,9 @@ class Window:
             'text': text
         }
 
-    def set_logo(self, path: str, position: str = '', alpha: float = -1) -> None:
+    def set_logo(self, path: str, position: str = '', alpha: float = -1, png_data: bytes = b'') -> None:
         path = resolve_custom_file(path) if path else ''
-        set_window_logo(self.os_window_id, self.tab_id, self.id, path, position or '', alpha)
+        set_window_logo(self.os_window_id, self.tab_id, self.id, path, position or '', alpha, png_data)
 
     def paste_with_actions(self, text: str) -> None:
         if self.destroyed or not text:
