@@ -11,10 +11,10 @@ var _ = fmt.Print
 
 var hostname string = "*"
 
-var Hostname = (&Once[string]{Run: func() string {
+var Hostname = Once(func() string {
 	h, err := os.Hostname()
 	if err == nil {
 		return h
 	}
 	return ""
-}}).Get
+})
